/* gr-mail.h:
 *
 * Copyright (C) 2017 Matthias Clasen <mclasen@redhat.com>
 *
 * Licensed under the GNU General Public License Version 3
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <gtk/gtk.h>

G_BEGIN_DECLS

void            gr_send_mail        (GtkWindow            *window,
                                     const char           *address,
                                     const char           *subject,
                                     const char           *body,
                                     const char          **attachments,
                                     GAsyncReadyCallback   callback,
                                     gpointer              user_data);
gboolean        gr_send_mail_finish (GAsyncResult         *result,
                                     GError              **error);

G_END_DECLS
